/*
 * CTRLS.c
 *
 *  Created on: 25.08.2019
 *      Author: alexanderbaumgartner
 *
 *  This file is part of ELoadV1_6.

    ELoadV1_6 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ELoadV1_6 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ELoadV1_6.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "../tivaHeaders.h"
#include "../Globals.h"
#include "CTRLS.h"
#include "../HAL/IOs.h"

#include <float.h>
#include <math.h>

extern main_t main_values;
extern spi_t spi_data;


void configureStdValues()
{
    main_values.cable_resistance = 0.0f;
    //Setting Values and limits to inital state
    main_values.lim.current = 3.5f;
    main_values.lim.voltage = 35.0f;
    main_values.lim.power = 95.0f;
    main_values.lim.uvlo = 5.0f;
    main_values.lim.t_sink = 50.0f;

    main_values.conversion.voltage_fac = 31.6f/2418.0f; //NOT ACCURATE!
    main_values.conversion.current_in_fac = 2.016/1623.0f; //NOT ACCURATE!
    main_values.conversion.current_out_fac = 1550.0f/2.00159645f; //NOT ACCURATE!
    main_values.conversion.temp_fac = (250.0f/4095.0f);
    main_values.act_filt.current_sum = 0.0f;
    main_values.act_filt.voltage_sum = 0.0f;
    main_values.act_filt.power_sum = 0.0f;
    main_values.act_filt.sinkTemp_sum = 0.0f;
    main_values.act_filt.summation_cnt = 1ul;
    main_values.act_filt.summation_clear = 0x00;
    main_values.act_filt.reading_block = 0x00;
    main_values.act_filt.writing_block = 0x00;

    main_values.set.current=0.0f;
    main_values.set.voltage=0.0f;
    main_values.set.power=0.0f;
    main_values.set.resistance=1000.0f;

    main_values.ctrl_mode = CTRL_MOD_NON;
    main_values.enable = LOAD_OFF;
    main_values.status = ERR_NO_ERR;

    main_values.curr_con.kI = 1.0f/F_CTRL;
    main_values.curr_con.kP = 0.0000f;
    main_values.curr_con.max = 0.02f;
    main_values.curr_con.min = -0.02f;
    main_values.curr_con.out_integrator = 0.0f;

    main_values.volt_con.kI = 1.0f/F_CTRL;
    main_values.volt_con.kP = 0.1f;
    main_values.volt_con.max = main_values.lim.current;
    main_values.volt_con.min = 0;
    main_values.volt_con.out_integrator = 0.0f;

    main_values.init_done = 0x00;

}

void calculateActValues()
{
    //calulation of the main.act values:
    main_values.act.current = ((float)spi_data.adc_val[0])*main_values.conversion.current_in_fac;
    main_values.act.voltage = ((float)spi_data.adc_val[1])*main_values.conversion.voltage_fac;

    //Add the voltage drop of the cables
    main_values.act.voltage += main_values.act.current * main_values.cable_resistance;


    main_values.act.power = main_values.act.current * main_values.act.voltage;
    if(main_values.act.voltage > 0.001f)
        main_values.act.resistance = main_values.act.voltage / main_values.act.current;
    else
        main_values.act.resistance = 10000.0f;//Very high value

    main_values.act.sinkTemp = ((float)spi_data.adc_val[3])*main_values.conversion.temp_fac;
}

void calculateFiltValues()
{
    //Old IIR Method
    /*const float filt_fact=0.995f, filt_fact_rev=1.0f-filt_fact;
    main_values.act_filt.current = filt_fact*main_values.act_filt.current + filt_fact_rev * main_values.act.current;
    main_values.act_filt.voltage = filt_fact*main_values.act_filt.voltage + filt_fact_rev * main_values.act.voltage;
    main_values.act_filt.power = filt_fact*main_values.act_filt.power + filt_fact_rev * main_values.act.power;
    main_values.act_filt.resistance = filt_fact*main_values.act_filt.resistance + filt_fact_rev * main_values.act.resistance;
    main_values.act_filt.sinkTemp = main_values.act.sinkTemp*filt_fact + main_values.act_filt.sinkTemp*filt_fact_rev;*/

    //New FIR (only calculate if the values are not read at the same time)
    if(main_values.act_filt.reading_block == 0x00)
    {
        main_values.act_filt.writing_block = 0xFF;
        if( (main_values.act_filt.summation_clear) || (main_values.act_filt.summation_cnt == 65536) )
        {
            //cathing an overflow
            main_values.act_filt.current_sum = (uint64_t) roundf(100000.0f * main_values.act.current);
            main_values.act_filt.voltage_sum = (uint64_t) roundf(100000.0f * main_values.act.voltage);
            main_values.act_filt.power_sum = (uint64_t) roundf(100000.0f * main_values.act.power);
            main_values.act_filt.sinkTemp_sum = (uint64_t) roundf(100.0f * main_values.act.sinkTemp);
            main_values.act_filt.summation_cnt=1ul;
            main_values.act_filt.summation_clear = 0x00;
        }
        else
        {
            main_values.act_filt.current_sum += (uint64_t) roundf(100000.0f * main_values.act.current);
            main_values.act_filt.voltage_sum += (uint64_t) roundf(100000.0f * main_values.act.voltage);
            main_values.act_filt.power_sum += (uint64_t) roundf(100000.0f * main_values.act.power);
            main_values.act_filt.sinkTemp_sum += (uint64_t) roundf(100.0f * main_values.act.sinkTemp);
            main_values.act_filt.summation_cnt++;
        }
        main_values.act_filt.writing_block = 0x00;
    }


}

void LoadCtrl()
{

    if(main_values.enable == LOAD_EN)
    {
        switch(main_values.ctrl_mode)
        {
        case CTRL_MOD_NON:
            //OFF State -> OK
            //Turn off the OPAMP
            spi_data.dac_val = 0;
            break;

        case CTRL_MOD_CURR:

            currentController();
            spi_data.dac_val = (uint16_t) roundf(main_values.conversion.current_out_fac*(main_values.set.current + main_values.curr_con.out));
            break;

        case CTRL_MOD_VOLT:

            voltageController();
            main_values.set.current = main_values.volt_con.out;
            spi_data.dac_val = (uint16_t) roundf(main_values.conversion.current_out_fac*main_values.set.current);

            break;

        case CTRL_MOD_PWR:
            main_values.set.current = main_values.set.power / main_values.act.voltage;
            if(main_values.set.current >= main_values.lim.current)
            {
                main_values.set.current = main_values.lim.current;
            }
            currentController();
            spi_data.dac_val = (uint16_t) roundf(main_values.conversion.current_out_fac*(main_values.set.current + main_values.curr_con.out));
            break;

        case CTRL_MOD_RES:
            main_values.set.current = main_values.act.voltage / main_values.set.resistance;

            if(main_values.set.current >= main_values.lim.current)
            {
                main_values.set.current = main_values.lim.current;
            }

            currentController();
            spi_data.dac_val = (uint16_t) roundf(main_values.conversion.current_out_fac*(main_values.set.current + main_values.curr_con.out));
            break;
        }
    }
    else
    {
        //Reset some values
        main_values.volt_con.out_integrator = 0.0f;
        main_values.curr_con.out_integrator = 0.0f;
    }
}


uint8_t checkLimits()
{
    uint8_t errs = ERR_NO_ERR;

    if(main_values.act.current > main_values.lim.current*1.1f)
        errs |= ERR_LIM_OVERCURR;
    if(main_values.act.voltage > main_values.lim.voltage*1.1f)
        errs |= ERR_LIM_OVERVOLT;
    if(main_values.act.power > main_values.lim.power*1.1f)
        errs |= ERR_LIM_OVERPOW;
    if(main_values.act.voltage < main_values.lim.uvlo)
        errs |= ERR_LIM_UVLO;
    if(main_values.act.sinkTemp > main_values.lim.t_sink)
        errs |= ERR_LIM_OVERTEMP;
    if(main_values.init_done == 0x00)
        errs |= ERR_NO_INIT;
    return errs;
}


void currentController()
{
    main_values.curr_con.out_integrator += (main_values.set.current - main_values.act.current) * main_values.curr_con.kI;
    if(main_values.curr_con.out_integrator > main_values.curr_con.max)
    {
        main_values.curr_con.out_integrator = main_values.curr_con.max;
    }
    else if(main_values.curr_con.out_integrator < main_values.curr_con.min)
    {
        main_values.curr_con.out_integrator = main_values.curr_con.min;
    }
    if(main_values.curr_con.out_integrator + main_values.set.current < 0.0f)
    {
        main_values.curr_con.out_integrator = 0.0f;
    }
    main_values.curr_con.out = main_values.curr_con.out_integrator;
}


void voltageController()
{

    main_values.volt_con.out = main_values.volt_con.kP*(main_values.act.voltage - main_values.set.voltage);//kP
    main_values.volt_con.out_integrator += (main_values.act.voltage - main_values.set.voltage)*main_values.volt_con.kI; //kI

    //Hard anti-windup for the integrator
    if(main_values.volt_con.out_integrator > main_values.volt_con.max)
    {
        main_values.volt_con.out_integrator = main_values.volt_con.max;
    }
    else if(main_values.volt_con.out_integrator < 0.0f)
    {
        main_values.volt_con.out_integrator=0.0f;
    }

    //Add kP and kI
    main_values.volt_con.out += main_values.volt_con.out_integrator;

    //Saturate
    if(main_values.volt_con.out >= main_values.volt_con.max)
    {
        main_values.volt_con.out = main_values.volt_con.max;
    }
    else if(main_values.volt_con.out < 0.0f)
    {
        main_values.volt_con.out=0.0f;
    }

}
