/*
 Author:    Alexander Baumgartner, ECE17
 Version:   1.6
 Title:     Electronic load - uC part

  *  This file is part of ELoadV1_6.

    ELoadV1_6 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ELoadV1_6 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ELoadV1_6.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "tivaHeaders.h"
#include "Globals.h"
#include "HAL/Clock.h"
#include "HAL/IOs.h"
#include "HAL/UART.h"
#include "HAL/SPI.h"
#include "DL/CTRLS.h"
extern main_t main_values;
extern uint32_t uptime;
int main(void)
{

    initClock();
    initSysTickSecondCounter();
    initIOs();
    initOPAMPSwitch();
    DisableOPAMP();

    FPUEnable();
    FPULazyStackingEnable(); //Enable floats in ISRs
    IntMasterEnable();//Global int enable

    initUART();
    initSPI2();
    delay_ms(1000.0f);
    writeLeds(PIN_RED_LED);

    configureStdValues();
    //Start the process
    statemachine_SPI_init_and_start();

    while(1)
    {
        delay_ms(0.1f);
        //System enabled and selected a mode
        if((main_values.enable > 0x00) && (main_values.ctrl_mode != CTRL_MOD_NON))
        {
            EnableOPAMP();
        }
        else
        {
            DisableOPAMP();
        }

        if(serial.rx_ok)
        {
            GPIOPinWrite(GPIO_PORTF_BASE,PIN_GREEN_LED,0xFF);
            GPIOPinWrite(GPIO_PORTF_BASE,PIN_RED_LED,0x00);
            serial.rx_ok=0x00;
            readDataUART();
            writeBackDataUART();
        }

        //check for RX-timeout
        if(uptime - serial.last_rx >1500) //1500ms -> 1.5s timeout
        {
            GPIOPinWrite(GPIO_PORTF_BASE,PIN_GREEN_LED|PIN_BLUE_LED,0x00);
            GPIOPinWrite(GPIO_PORTF_BASE,PIN_RED_LED,0xFF);
            main_values.status|=ERR_COM_TIMEOUT;
        }
    }
}



//*****************************************************************************
//
// The error routine that is called if the driver library encounters an error.
//
//*****************************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, uint32_t ui32Line)
{
}
#endif
