/*
 * Globals.h
 *
 *  Created on: 16.06.2019
 *      Author: alexanderbaumgartner
 *
 *  This file is part of ELoadV1_6.

    ELoadV1_6 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ELoadV1_6 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ELoadV1_6.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GLOBALS_H_
#define GLOBALS_H_
/*
 * Config
 * */
#define UART_BAUD   115200ul
/*
 * Pins
 * */
//Port A
#define PIN_OPAMP_EN  GPIO_PIN_7

//Port F
#define PIN_RED_LED   GPIO_PIN_1
#define PIN_BLUE_LED  GPIO_PIN_2
#define PIN_GREEN_LED GPIO_PIN_3


/* Variables and types*/

uint32_t uptime;

typedef struct
{
    uint32_t send_val;
    uint32_t rec_val[3];
    uint16_t dac_val;
    uint16_t adc_val[4];
    uint8_t state_step;
    uint8_t state_curr_or_temp;
} spi_t;



typedef struct
{
    float current;
    float voltage;
    float power;
    float uvlo;
    float t_sink;
} main_limits_t;

typedef struct
{
    float current;
    float voltage;
    float power;
    float resistance;
    float sinkTemp;
} main_act_t;

typedef struct
{
    uint64_t current_sum;
    uint64_t voltage_sum;
    uint64_t power_sum;
    uint64_t sinkTemp_sum;
    uint64_t summation_cnt;
    uint8_t summation_clear;
    uint8_t reading_block;
    uint8_t writing_block;
    float current;
    float voltage;
    float power;
    float resistance;
    float sinkTemp;
} main_act_filt_t;

typedef struct
{
    float current;
    float voltage;
    float power;
    float resistance;
} main_set_t;

typedef struct
{
    float current_in_fac;
    float voltage_fac;
    float current_out_fac;
    float temp_fac;
} value_conv_t;

typedef struct
{
    float kP;
    float kI;
    float max;
    float min;
    float out;
    float out_integrator;
} main_contr_t;

typedef struct
{
    main_act_t act;
    main_act_filt_t act_filt;
    main_set_t set;
    main_limits_t lim;
    main_contr_t curr_con;
    main_contr_t volt_con;
    value_conv_t conversion;
    float cable_resistance;
    uint8_t ctrl_mode;
    uint8_t status;
    uint8_t enable;
    uint8_t init_done;
} main_t;


main_t main_values;
spi_t spi_data;


#endif /* GLOBALS_H_ */
