/*
 * SPI.c
 *
 *  Created on: 19.06.2019
 *      Author: alexanderbaumgartner
 *
 *  This file is part of ELoadV1_6.

    ELoadV1_6 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ELoadV1_6 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ELoadV1_6.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../tivaHeaders.h"
#include "../Globals.h"
#include "../DL/CTRLS.h"
#include "SPI.h"

extern main_t main;

void initSPI2()
{
    //Set the CS Pin as output
    GPIOPinTypeGPIOOutput(SPI2_CS1_BASE, SPI2_CS1_PIN);
    //This one is active low -> set it to a high
    GPIOPinWrite(SPI2_CS1_BASE, SPI2_CS1_PIN, 0xFF);
    //Set the CS Pin as output
    GPIOPinTypeGPIOOutput(SPI2_CS1_BASE, SPI2_CS2_PIN);
    //This one is active low -> set it to a high
    GPIOPinWrite(SPI2_CS1_BASE, SPI2_CS2_PIN, 0xFF);
    //Enable the SSI0 module
    SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI2);
    // Wait for the SSI0 module to be ready
    while(!SysCtlPeripheralReady(SYSCTL_PERIPH_SSI2))
    {;}

    //GPIO Pin Config
    GPIOPinTypeGPIOOutput(GPIO_PORTB_BASE, GPIO_PIN_7 | GPIO_PIN_5 |GPIO_PIN_4);
    GPIOPinTypeGPIOInput(GPIO_PORTB_BASE, GPIO_PIN_6);
    GPIOPinConfigure(GPIO_PB4_SSI2CLK);
    GPIOPinConfigure(GPIO_PB5_SSI2FSS);
    GPIOPinConfigure(GPIO_PB6_SSI2RX);
    GPIOPinConfigure(GPIO_PB7_SSI2TX);

    //Enable the pins
    GPIOPinTypeSSI(GPIO_PORTB_BASE, GPIO_PIN_7 | GPIO_PIN_6 | GPIO_PIN_5 |GPIO_PIN_4);
    //Configure the SPI (Pol:0,Phase:0)
    SSIConfigSetExpClk(SSI2_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_0,SSI_MODE_MASTER, SPI2_SPEED, 16);

    //Enable the interface
    SSIEnable(SSI2_BASE);

    //Enable the interrupts
    // SPI_inter_handler function will be an interrupt handler
    IntRegister(INT_SSI2, ISR_SPI2);
    SSIIntEnable(SSI2_BASE,SSI_RXFF|SSI_RXTO|SSI_RXOR);

    // SPI interrupt enable
    IntEnable(INT_SSI2);
}


void writeDACValue()
{
    uint32_t val = (uint32_t)spi_data.dac_val;
    //limit the value
    if(val > 4095)
    {
        val=4095;
    }
    //Clear the upper 4 bits
    //val = val & 0x0FFF;

    //adding the configuration to the word
    val |= (1<<12) | (DAC_CONF_GAIN<<13) | (DAC_CONF_VREF_BUF<<14);

    //writing the data
    GPIOPinWrite(SPI2_CS1_BASE, SPI2_CS1_PIN, 0x00);
    SSIDataPut(SSI2_BASE, val);
}

void readADCValue(uint8_t channel)
{
    uint32_t tmp = 0x600ul | ((uint32_t)channel << 6);
    //writing the data
    GPIOPinWrite(SPI2_CS1_BASE, SPI2_CS2_PIN, 0x00);
    SSIDataPut(SSI2_BASE, tmp);
    SSIDataPut(SSI2_BASE, 0ul);
}


void ISR_SPI2()
{
    SSIIntClear(SSI2_BASE,SSI_TXFF| SSI_RXFF);
    if(!SSIBusy(SSI2_BASE))
    {
        statemachine_SPI();
    }
}


void statemachine_SPI_init_and_start()
{
    spi_data.state_step=0;
    spi_data.dac_val=0;
    statemachine_SPI();
}


void statemachine_SPI()
{
    //Reset the CS Pins
    GPIOPinWrite(SPI2_CS1_BASE, SPI2_CS2_PIN, 0xFF);
    GPIOPinWrite(SPI2_CS1_BASE, SPI2_CS1_PIN, 0xFF);
    switch(spi_data.state_step)
    {
    case 0:
        /**Startup-case -> Trigger ADC Ch.0**/
        readADCValue(0);

        //Triggering next step -> Read Ch.0
        spi_data.state_step=1;
        spi_data.state_curr_or_temp = 0;
        break;
    case 1:
        /**Read ADC Ch. 0 or 3 (Current or Temp)-> Trigger ADC Ch.1 (Voltage)**/
        //Read ADC:
        //current
        if((spi_data.state_curr_or_temp % 2) == 0)
        {
            SSIDataGet(SSI2_BASE,&spi_data.rec_val[0]);
            SSIDataGet(SSI2_BASE,&spi_data.rec_val[1]);
            spi_data.adc_val[0] = ((uint16_t)spi_data.rec_val[0]&0x000F)<<8;
            spi_data.adc_val[0] |= ((uint16_t)(spi_data.rec_val[1]&0xFF00)>>8);
        }
        else //Temperature
        {
            SSIDataGet(SSI2_BASE,&spi_data.rec_val[0]);
            SSIDataGet(SSI2_BASE,&spi_data.rec_val[1]);
            spi_data.adc_val[3] = ((uint16_t)spi_data.rec_val[0]&0x000F)<<8;
            spi_data.adc_val[3] |= ((uint16_t)(spi_data.rec_val[1]&0xFF00)>>8);
        }
        spi_data.state_curr_or_temp++;

        //Triggering next step -> Read Ch.1
        readADCValue(1);
        spi_data.state_step=2;
        break;
    case 2:
        /**Read ADC Ch. 1 (Voltage)**/
        //Read ADC:
        SSIDataGet(SSI2_BASE,&spi_data.rec_val[0]);
        SSIDataGet(SSI2_BASE,&spi_data.rec_val[1]);
        spi_data.adc_val[1] = ((uint16_t)spi_data.rec_val[0]&0x000F)<<8;
        spi_data.adc_val[1] |= ((uint16_t)(spi_data.rec_val[1]&0xFF00)>>8);

        //Triggering Writing Data to the DAC
        writeDACValue();
        spi_data.state_step=3;
        break;
    case 3:
        /**Throw away DAC Read -> Trigger ADC Ch.0**/
        SSIDataGet(SSI2_BASE,&spi_data.rec_val[2]);

        //Triggering next step -> Read Ch.0 or 3 (curr or temp)
        if((spi_data.state_curr_or_temp % 2) == 0)
            readADCValue(0);
        else
            readADCValue(3);
        spi_data.state_step=1;
        break;
    }
    if(spi_data.state_step==3)
    {
        calculateActValues();
        main_values.status|=checkLimits();
        if(main_values.status!=ERR_NO_ERR)
            main_values.enable=0x00;
        calculateFiltValues();
        LoadCtrl();
    }


}

